"""
Include RAG (Retrieval-Augmented Generation) search functionality for specific Python libraries
(search_python_libs, LibraryInfoExtractor, PythonDocumentationProcessor).

As well as for broader use (search_documents, HashingVectorizerSearch, TfidfSearch, SentenceTransformerSearch, EnsembleSearchEngine).
"""
